@@ -1,7 +1,7 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //========================================================================
 //
-// Copyright (C) 2002-2006 Yves Renard, Julien Pommier.
+// Copyright (C) 2002-2007 Yves Renard, Julien Pommier.
 //
 // This file is a part of GETFEM++
 //
@@ -27,18 +27,22 @@
  * a good example of use of Getfem++.
 */
 
-#include <getfem_assembling.h> /* import assembly methods (and norms comp.) */
-#include <getfem_export.h>   /* export functions (save solution in a file)  */
-#include <getfem_derivatives.h>
-#include <getfem_regular_meshes.h>
-#include <getfem_model_solvers.h>
-#include <getfem_mesh_im_level_set.h>
-#include <getfem_mesh_fem_level_set.h>
-#include <getfem_mesh_fem_product.h>
-#include <getfem_mesh_fem_global_function.h>
-#include <getfem_spider_fem.h>
-#include <getfem_mesh_fem_sum.h>
-#include <gmm.h>
+#include "getfem/getfem_assembling.h" /* import assembly methods (and norms comp.) */
+#include "getfem/getfem_export.h"   /* export functions (save solution in a file)  */
+#include "getfem/getfem_derivatives.h"
+#include "getfem/getfem_regular_meshes.h"
+#include "getfem/getfem_model_solvers.h"
+#include "getfem/getfem_mesh_im_level_set.h"
+#include "getfem/getfem_mesh_fem_level_set.h"
+#include "getfem/getfem_mesh_fem_product.h"
+#include "getfem/getfem_mesh_fem_global_function.h"
+#include "getfem/getfem_spider_fem.h"
+#include "getfem/getfem_mesh_fem_sum.h"
+#include "gmm/gmm.h"
+#include "getfem/getfem_error_estimate.h"
+#include <sstream>
+#include "getfem/getfem_interpolated_fem.h"
+
 
 /* some Getfem++ types that we will be using */
 using bgeot::base_small_vector; /* special class for small (dim<16) vectors */
@@ -47,7 +51,8 @@ using bgeot::scalar_type; /* = double */
 using bgeot::size_type;   /* = unsigned long */
 using bgeot::base_matrix; /* small dense matrix. */
 
-/* definition of some matrix/vector types. These ones are built
+/* definition of some matrix/vector types. These ones are builtmayavi -d crack.vtk -f WarpVector -m BandedSurfaceMap -m Outline
+
  * using the predefined types in Gmm++
  */
 typedef getfem::modeling_standard_sparse_vector sparse_vector;
@@ -240,10 +245,16 @@ struct exact_solution {
   
   void init(int mode, scalar_type lambda, scalar_type mu,
 	    getfem::level_set &ls) {
-    std::vector<getfem::pglobal_function> cfun(4);
-    for (unsigned j=0; j < 4; ++j)
-      cfun[j] = getfem::isotropic_crack_singular_2D(j, ls);
-    mf.set_functions(cfun);
+    std::vector<getfem::pglobal_function> cfunc(4);
+   
+ for (size_type i = 0; i < 4; ++i) {
+    /* use the singularity */
+    getfem::abstract_xy_function *s = 
+      new getfem::crack_singular_xy_function(i);
+    cfunc[i] = getfem::global_function_on_level_set(ls, *s);
+ }
+
+    mf.set_functions(cfunc);
     
     mf.set_qdim(1);
    
@@ -310,7 +321,7 @@ struct crack_problem {
   getfem::mesh_fem_sum mf_u_sum;
   
   getfem::mesh_fem& mf_u() { return mf_u_sum; }
-    
+  
   scalar_type lambda, mu;    /* Lame coefficients.                */
   getfem::mesh_fem mf_rhs;   /* mesh_fem for the right hand side (f(x),..)   */
   
@@ -323,24 +334,27 @@ struct crack_problem {
   int bimaterial;           /* For bimaterial interface fracture */
   bool all_dirichlet;
   double F11,F12,F21,F22,F31,F32,F41,F42;       /* NEUMANN forces */
-  double lambda_up, lambda_down;  /*Lame coeff for bimaterial case*/
+  double neumann_value;   /* NEUMANN force value */
+  int mode;
+  double lambda_up, lambda_down, mu_up, mu_down;  /*Lame coeff for bimaterial case*/
   getfem::level_set ls;      /* The two level sets defining the crack.       */
   
   base_small_vector translation;
 
   scalar_type residual;       /* max residual for the iterative solvers        */
-
+  scalar_type conv_max;
   unsigned dir_with_mult;
   
   std::string datafilename;
-  ftool::md_param PARAM;
-
-  bool solve(plain_vector &U);
+  bgeot::md_param PARAM;
+  
+  bool adapted_refine, solve(plain_vector &U);
+  
   void init(void);
   crack_problem(void) : mls(mesh), mim(mls), mf_pre_u(mesh), mf_mult(mesh),
 			mfls_u(mls, mf_pre_u),
 			
-			mf_u_sum(mesh), mf_rhs(mesh),
+			mf_u_sum(mesh), mf_rhs(mesh), 
 #ifdef VALIDATE_XFEM
 			exact_sol(mesh), 
 #endif
@@ -354,6 +368,7 @@ struct crack_problem {
 void crack_problem::init(void) {
   std::string MESH_TYPE = PARAM.string_value("MESH_TYPE","Mesh type ");
   std::string FEM_TYPE  = PARAM.string_value("FEM_TYPE","FEM name");
+
   std::string INTEGRATION = PARAM.string_value("INTEGRATION",
 					       "Name of integration method");
   std::string SIMPLEX_INTEGRATION = PARAM.string_value("SIMPLEX_INTEGRATION",
@@ -385,31 +400,68 @@ void crack_problem::init(void) {
   M(1,1) = ly;
   mesh.transformation(M);
   
-  base_small_vector tt(N); tt[1] = -(lx/2.);
+  base_small_vector tt(N); tt[0] = tt[1] = -(lx/2.);
   mesh.translation(tt); 
 
+  conv_max = PARAM.int_value("CONV_MAX","Maximal number of convexes in the mesh");
+  adapted_refine = PARAM.int_value("ADAPTED_REFINE","Adapted Refinement");
   
+  scalar_type refinement_radius;
+  refinement_radius = PARAM.real_value("REFINEMENT_RADIUS","Refinement Radius");
+  size_type refinement_process;
+  refinement_process = PARAM.int_value("REFINEMENT_PROCESS","Refinement process");
+  dal::bit_vector conv_to_refine;
+  size_type ref = 1;  
+  if (refinement_radius > 0){
+    while(ref <= refinement_process){
+      conv_to_refine.clear();
+      for(dal::bv_visitor i(mesh.convex_index()); !i.finished(); ++i){
+      for(size_type j=0; j < 3; ++j)
+	if(fabs(mesh.points()[mesh.ind_points_of_convex(i)[j]][0])<refinement_radius 
+	   && fabs(mesh.points()[mesh.ind_points_of_convex(i)[j]][1])<refinement_radius){
+	  conv_to_refine.add(i);
+	}
+      }
+      mesh.Bank_refine(conv_to_refine);
+      
+      ref = ref + 1; 
+      refinement_radius = refinement_radius/2.;
+      if(refinement_radius > 1e-16 )
+	cout<<"refining process step " << ref << "... refining "<< conv_to_refine.size() <<" convexes..." << endl ; 
+      
+    }
+  cout<<"refining process complete." << endl ;
+  }
   datafilename = PARAM.string_value("ROOTFILENAME","Base name of data files.");
+  residual = PARAM.real_value("RESIDUAL");
+  if (residual == 0.) residual = 1e-10;
   
   bimaterial = PARAM.int_value("BIMATERIAL", "Bimaterial interface crack");
   all_dirichlet = PARAM.int_value("all_dirichlet", "Dirichlet condition");
-  F11 = PARAM.real_value("F11","F11");
-  F12 = PARAM.real_value("F12","F12");
-  F21 = PARAM.real_value("F21","F21");
-  F22 = PARAM.real_value("F22","F22");
-  F31 = PARAM.real_value("F31","F31");
-  F32 = PARAM.real_value("F32","F32");
-  F41 = PARAM.real_value("F41","F41");
-  F42 = PARAM.real_value("F42","F42");
+  mode = PARAM.int_value("mode","mode");
+
+  //F11 = PARAM.real_value("F11","F11");
+  //F12 = PARAM.real_value("F12","F12");
+  //F21 = PARAM.real_value("F21","F21");
+  //F22 = PARAM.real_value("F22","F22");
+  //F31 = PARAM.real_value("F31","F31");
+  //F32 = PARAM.real_value("F32","F32");
+  //F41 = PARAM.real_value("F41","F41");
+  //F42 = PARAM.real_value("F42","F42");
  
+  neumann_value = PARAM.real_value("NEUMANN_VALUE","neumann_value");
+
   if (bimaterial == 1){
     mu = PARAM.real_value("MU", "Lame coefficient mu"); 
     lambda_up = PARAM.int_value("LAMBDA_UP", "Lame Coef");
     lambda_down = PARAM.int_value("LAMBDA_DOWN", "Lame Coef");
     lambda = PARAM.real_value("LAMBDA", "Lame coefficient lambda");
+    mu_up = PARAM.int_value("MU_UP", "Lame Coef");
+    mu_down = PARAM.int_value("MU_DOWN", "Lame Coef");
+    
   }
   else{
-
+    
     mu = PARAM.real_value("MU", "Lame coefficient mu");
     lambda = PARAM.real_value("LAMBDA", "Lame coefficient lambda");
   }
@@ -417,9 +469,12 @@ void crack_problem::init(void) {
 
   mf_u().set_qdim(N);
 
+
   /* set the finite element on the mf_u */
   getfem::pfem pf_u = 
     getfem::fem_descriptor(FEM_TYPE);
+ 
+ 
   getfem::pintegration_method ppi = 
     getfem::int_method_descriptor(INTEGRATION);
   getfem::pintegration_method simp_ppi = 
@@ -434,18 +489,17 @@ void crack_problem::init(void) {
   mf_pre_u.set_finite_element(mesh.convex_index(), pf_u);
   mf_mult.set_finite_element(mesh.convex_index(), pf_u);
   mf_mult.set_qdim(N);
-   
+
+
   dir_with_mult = PARAM.int_value("DIRICHLET_VERSINO");
  
   /* set the finite element on mf_rhs (same as mf_u is DATA_FEM_TYPE is
      not used in the .param file */
   std::string data_fem_name = PARAM.string_value("DATA_FEM_TYPE");
   if (data_fem_name.size() == 0) {
-    if (!pf_u->is_lagrange()) {
-      DAL_THROW(dal::failure_error, "You are using a non-lagrange FEM. "
+    GMM_ASSERT1(pf_u->is_lagrange(), "You are using a non-lagrange FEM. "
 		<< "In that case you need to set "
 		<< "DATA_FEM_TYPE in the .param file");
-    }
     mf_rhs.set_finite_element(mesh.convex_index(), pf_u);
   } else {
     mf_rhs.set_finite_element(mesh.convex_index(), 
@@ -505,8 +559,8 @@ base_small_vector ls_function(const base_node P, int num = 0) {
   base_small_vector res(2);
   switch (num) {
     case 0: {
-      res[0] = y;
-      res[1] = -(0.1) + x;
+      res[0] =  y;
+      res[1] =  x;
     } break;
     case 1: {
       res[0] = gmm::vect_dist2(P, base_node(0.5, 0.)) - .25;
@@ -522,136 +576,434 @@ base_small_vector ls_function(const base_node P, int num = 0) {
 }
 
 bool crack_problem::solve(plain_vector &U) {
-  size_type nb_dof_rhs = mf_rhs.nb_dof();
   size_type N = mesh.dim();
-  ls.reinit();  
-  cout << "ls.get_mesh_fem().nb_dof() = " << ls.get_mesh_fem().nb_dof() << "\n";
-  for (size_type d = 0; d < ls.get_mesh_fem().nb_dof(); ++d) {
-    ls.values(0)[d] = ls_function(ls.get_mesh_fem().point_of_dof(d), 0)[0];
-    ls.values(1)[d] = ls_function(ls.get_mesh_fem().point_of_dof(d), 0)[1];
-  }
-  ls.touch();
-  mls.adapt();
-  mim.adapt();
-  mfls_u.adapt();
- 
-  mf_u_sum.set_mesh_fems(mfls_u);
   
   
-
-  U.resize(mf_u().nb_dof());
-
-  
   // Linearized elasticity brick.
+  dal::bit_vector conv_to_refine;
+  bool iteration(false);
   
-  
-  getfem::mdbrick_isotropic_linearized_elasticity<>
-    ELAS(mim, mf_u(), lambda, mu);
-
-  
-  if(bimaterial == 1){
-    cout<<"______________________________________________________________________________"<<endl;
-    cout<<"CASE OF BIMATERIAL CRACK  with lambda_up = "<<lambda_up<<" and lambda_down = "<<lambda_down<<endl;
-    cout<<"______________________________________________________________________________"<<endl;
-    std::vector<float> bi_lambda(ELAS.lambda().mf().nb_dof());
+  switch (mode) {
+  case 1:
+  case 2:{
+    if (mode == 1){
+      cout << "Mode I problem..." << endl;
+      F11 = 0; F12 = neumann_value; F21 = 0; F22 = 0; F31 = 0; F32 = 0; F41 = 0; F42 = -neumann_value;
+    } else 
+      if(mode == 2){
+	cout << "Mode II problem..." << endl;
+	F11 = 0; F12 = 0; F21 = neumann_value; F22 = 0; F31 = -neumann_value; F32 = 0; F41 = 0; F42 = 0;
+      }
     
-    cout<<"ELAS.lambda().mf().nb_dof()==="<<ELAS.lambda().mf().nb_dof()<<endl;
-    
-    for (size_type ite = 0; ite < ELAS.lambda().mf().nb_dof();ite++) {
-      if (ELAS.lambda().mf().point_of_dof(ite)[1] > 0)
-	bi_lambda[ite] = lambda_up;
+    do {
+      cout << "Number of convexes : "<<  mesh.convex_index().size() <<endl;
+      size_type nb_dof_rhs = mf_rhs.nb_dof();
+      ls.reinit();  
+      cout << "ls.get_mesh_fem().nb_dof() = " << ls.get_mesh_fem().nb_dof() << "\n";
+      for (size_type d = 0; d < ls.get_mesh_fem().nb_dof(); ++d) {
+	ls.values(0)[d] = ls_function(ls.get_mesh_fem().point_of_dof(d), 0)[0];
+	ls.values(1)[d] = ls_function(ls.get_mesh_fem().point_of_dof(d), 0)[1];
+      }
+      ls.touch();
+      mls.adapt();
+      mim.adapt();
+      mfls_u.adapt();
+      
+      mf_u_sum.set_mesh_fems(mfls_u);
+      
+      U.resize(mf_u().nb_dof());
+      
+      conv_to_refine.clear();
+      getfem::mdbrick_isotropic_linearized_elasticity<>
+	ELAS(mim, mf_u(), lambda, mu);
+      
+      
+      if(bimaterial == 1){
+	cout<<"______________________________________________________________________________"<<endl;
+	cout<<"CASE OF BIMATERIAL CRACK  with lambda_up = "<<lambda_up<<" and lambda_down = "<<lambda_down<<endl;
+	cout<<"______________________________________________________________________________"<<endl;
+	std::vector<float> bi_lambda(ELAS.lambda().mf().nb_dof());
+	std::vector<float> bi_mu(ELAS.lambda().mf().nb_dof());
+	
+	cout<<"ELAS.lambda().mf().nb_dof()==="<<ELAS.lambda().mf().nb_dof()<<endl;
+	
+	for (size_type ite = 0; ite < ELAS.lambda().mf().nb_dof();ite++) {
+	  if (ELAS.lambda().mf().point_of_dof(ite)[1] > 0){
+	    bi_lambda[ite] = lambda_up;
+	    bi_mu[ite] = mu_up;
+	  }
+	  else{
+	    bi_lambda[ite] = lambda_down;
+	    bi_mu[ite] = mu_down;
+	  }
+	} 
+	//cout<<"bi_lambda.size() = "<<bi_lambda.size()<<endl;
+	// cout<<"ELAS.lambda().mf().nb_dof()==="<<ELAS.lambda().mf().nb_dof()<<endl;
+	
+	ELAS.lambda().set(bi_lambda);
+	ELAS.mu().set(bi_mu);
+      }
+      
+      
+      // Defining the volumic source term.
+      
+      
+      plain_vector F(nb_dof_rhs * N);
+      for (size_type i = 0; i < nb_dof_rhs; ++i)
+	gmm::copy(sol_f(mf_rhs.point_of_dof(i)),
+		  gmm::sub_vector(F, gmm::sub_interval(i*N, N)));
+      
+      // Volumic source term brick.
+      getfem::mdbrick_source_term<> VOL_F(ELAS, mf_rhs, F);
+      
+      // Defining the Neumann condition right hand side.
+      
+      // Neumann condition brick.
+      
+      getfem::mdbrick_abstract<> *pNEUMANN;
+      
+      gmm::clear(F);
+      for(size_type i = 0; i<F.size(); i=i+2) 
+	{F[i] = F41; F[i+1] = F42;}
+      
+      getfem::mdbrick_source_term<> NEUMANN_down(VOL_F, mf_rhs, F,NEUMANN_BOUNDARY_NUM_down);
+      
+      gmm::clear(F);
+      for(size_type i = 0; i<F.size(); i=i+2) 
+	{F[i] = F21; F[i+1] = F22;}
+      
+      getfem::mdbrick_source_term<> NEUMANN_right_up(NEUMANN_down, mf_rhs, F,NEUMANN_BOUNDARY_NUM_right);
+      
+      
+      gmm::clear(F);
+      for(size_type i = 0; i<F.size(); i=i+2) 
+	{F[i] = F31; F[i+1] = F32;}
+      
+      getfem::mdbrick_source_term<> NEUMANN_right_down(NEUMANN_right_up, mf_rhs, F,NEUMANN_BOUNDARY_NUM_right+1);
+      
+      gmm::clear(F);
+      for(size_type i = 0; i<F.size(); i=i+2) 
+	{F[i] = F11; F[i+1] = F12;}
+      
+      getfem::mdbrick_source_term<> NEUMANN_up(NEUMANN_right_down, mf_rhs, F,NEUMANN_BOUNDARY_NUM_up);
+      
+      if (all_dirichlet){
+	cout << "Exact -DIRICHLET- Mode I problem..." << endl;
+	pNEUMANN = & VOL_F;
+      }
+      else 
+	pNEUMANN = & NEUMANN_up; 
+      
+      //toto_solution toto(mf_rhs.linked_mesh()); toto.init();
+      //assert(toto.mf.nb_dof() == 1);
+      
+      // Dirichlet condition brick.
+      getfem::mdbrick_Dirichlet<> final_model(*pNEUMANN, DIRICHLET_BOUNDARY_NUM,
+					      mf_mult);
+      if(all_dirichlet){
+#ifdef VALIDATE_XFEM
+	final_model.rhs().set(exact_sol.mf,exact_sol.U);
+#endif
+      } else {
+#ifdef VALIDATE_XFEM
+	final_model.rhs().set(exact_sol.mf,0);
+#endif
+      }
+      final_model.set_constraints_type(getfem::constraints_type(dir_with_mult));
+      
+      // Generic solve.
+      cout << "Total number of variables : " << final_model.nb_dof() << endl;
+      getfem::standard_model_state MS(final_model);
+      gmm::iteration iter(residual, 1, 40000);
+      
+      //getfem::standard_solve(MS, final_model, iter);
+      getfem::standard_solve(MS, final_model, iter, getfem::select_linear_solver(final_model, "superlu"));
+      
+      // Solution extraction
+      gmm::copy(ELAS.get_solution(MS), U);
+      iteration = iter.converged();  
+      
+      // Adapted Refinement (suivant une erreur a posteriori)
+      if (adapted_refine) {
+	if (mesh.convex_index().size() < conv_max){
+	  plain_vector ERR(mesh.convex_index().last_true()+1);
+	  getfem::error_estimate(mim, mf_u(), U, ERR);
+	  
+	  cout << "max = " << gmm::vect_norminf(ERR) << endl;
+	  scalar_type threshold = 0.01, min_ = 1e18;
+	  conv_to_refine.clear();
+	  for (dal::bv_visitor i(mesh.convex_index()); !i.finished(); ++i) {
+	    if (ERR[i] > threshold && mesh.points()[mesh.ind_points_of_convex(i)[0]][0] >= -0.1) 
+	      conv_to_refine.add(i);
+	    min_ = std::min(min_, ERR[i]);
+	  }
+	  cout << "min = " << min_ << endl;
+      cout << "Refining " <<  conv_to_refine.card() << " convexes..."<< endl;  
+      mesh.Bank_refine(conv_to_refine);
+	}    
 	else
-	  bi_lambda[ite] = lambda_down;
-    }
+	  break;
+      }
+    } while(adapted_refine && conv_to_refine.size() > 0);
+    mesh.write_to_file(datafilename + ".meshh");
+    cout << "Refining process complete. The mesh contains now " <<  mesh.convex_index().size() << " convexes "<<endl;
     
-    //cout<<"bi_lambda.size() = "<<bi_lambda.size()<<endl;
-    // cout<<"ELAS.lambda().mf().nb_dof()==="<<ELAS.lambda().mf().nb_dof()<<endl;
+    dal::bit_vector blocked_dof = mf_u().dof_on_set(5);
+    getfem::mesh_fem mf_printed(mesh, N);
+    std::string FEM_DISC = PARAM.string_value("FEM_DISC","fem disc ");
+    mf_printed.set_finite_element(mesh.convex_index(),
+				  getfem::fem_descriptor(FEM_DISC));
     
-    ELAS.lambda().set(bi_lambda);
-  }
-  
+    plain_vector W(mf_printed.nb_dof());
+    getfem::interpolation(mf_u(), mf_printed, U, W);
 
-  // Defining the volumic source term.
-  plain_vector F(nb_dof_rhs * N);
-  for (size_type i = 0; i < nb_dof_rhs; ++i)
-      gmm::copy(sol_f(mf_rhs.point_of_dof(i)),
-		gmm::sub_vector(F, gmm::sub_interval(i*N, N)));
-  
-  // Volumic source term brick.
-  getfem::mdbrick_source_term<> VOL_F(ELAS, mf_rhs, F);
+    std::ostringstream oss;
+    oss << mode;
+    std::string mode_string = oss.str();
 
-  // Defining the Neumann condition right hand side.
-    
-  // Neumann condition brick.
-  
-  getfem::mdbrick_abstract<> *pNEUMANN;
+    mf_printed.write_to_file(datafilename + mode_string +  ".meshfem", true);
+    gmm::vecsave(datafilename + mode_string + ".U", W);
 
-  gmm::clear(F);
-  for(size_type i = 0; i<F.size(); i=i+2) 
-    {F[i] = F41; F[i+1] = F42;}
-  
-  getfem::mdbrick_source_term<> NEUMANN_down(VOL_F, mf_rhs, F,NEUMANN_BOUNDARY_NUM_down);
-  
-  gmm::clear(F);
-  for(size_type i = 0; i<F.size(); i=i+2) 
-    {F[i] = F21; F[i+1] = F22;}
-  
-  getfem::mdbrick_source_term<> NEUMANN_right_up(NEUMANN_down, mf_rhs, F,NEUMANN_BOUNDARY_NUM_right);
-  
-  
-  gmm::clear(F);
-  for(size_type i = 0; i<F.size(); i=i+2) 
-     {F[i] = F31; F[i+1] = F32;}
-  
-  getfem::mdbrick_source_term<> NEUMANN_right_down(NEUMANN_right_up, mf_rhs, F,NEUMANN_BOUNDARY_NUM_right+1);
-  
-  gmm::clear(F);
-  for(size_type i = 0; i<F.size(); i=i+2) 
-    {F[i] = F11; F[i+1] = F12;}
-  
-  getfem::mdbrick_source_term<> NEUMANN_up(NEUMANN_right_down, mf_rhs, F,NEUMANN_BOUNDARY_NUM_up);
-  
-  if (all_dirichlet)
-    pNEUMANN = & VOL_F; 
-  else 
-    pNEUMANN = & NEUMANN_up; 
+  }break;
     
-  //toto_solution toto(mf_rhs.linked_mesh()); toto.init();
-  //assert(toto.mf.nb_dof() == 1);
-  
-  // Dirichlet condition brick.
-  getfem::mdbrick_Dirichlet<> final_model(*pNEUMANN, DIRICHLET_BOUNDARY_NUM,
-					  mf_mult);
-  if(all_dirichlet){
+  case 12:{
+    int mode_counter = 1;
+    plain_vector W12;
+    W12.clear();
+    getfem::mesh_fem mf_printed(mesh, N);
+    dal::bit_vector blocked_dof;
+    
+    std::string FEM_DISC = PARAM.string_value("FEM_DISC","fem disc ");
+
+    do{
+      cout << "Computing mode " << mode_counter << " solution..." <<endl;
+      
+      if (mode_counter == 1){
+	F11 = 0; F12 = neumann_value; F21 = 0; F22 = 0; F31 = 0; F32 = 0; F41 = 0; F42 = -neumann_value;
+	//F11 = 0; F12 = 0; F21 = neumann_value; F22 = 0; F31 = -neumann_value; F32 = 0; F41 = 0; F42 = 0;
+      } else 
+	
+	if(mode_counter == 2){
+	  F11 = 0; F12 = 0; F21 = neumann_value; F22 = 0; F31 = -neumann_value; F32 = 0; F41 = 0; F42 = 0;
+	  //F11 = 0; F12 = neumann_value; F21 = 0; F22 = 0; F31 = 0; F32 = 0; F41 = 0; F42 = -neumann_value;
+	}
+
+      do {
+	cout << "Number of convexes : "<<  mesh.convex_index().size() <<endl;
+	size_type nb_dof_rhs = mf_rhs.nb_dof();
+
+	if (mode_counter == 1) {
+	  ls.reinit();  
+	  cout << "ls.get_mesh_fem().nb_dof() = " << ls.get_mesh_fem().nb_dof() << "\n";
+	  for (size_type d = 0; d < ls.get_mesh_fem().nb_dof(); ++d) {
+	    ls.values(0)[d] = ls_function(ls.get_mesh_fem().point_of_dof(d), 0)[0];
+	    ls.values(1)[d] = ls_function(ls.get_mesh_fem().point_of_dof(d), 0)[1];
+	  }
+	  ls.touch();
+	  mls.adapt();
+	  mim.adapt();
+	  mfls_u.adapt();	  
+	  mf_u_sum.set_mesh_fems(mfls_u);
+	  U.resize(mf_u().nb_dof());
+	}
+
+	conv_to_refine.clear();
+	getfem::mdbrick_isotropic_linearized_elasticity<>
+	  ELAS(mim, mf_u(), lambda, mu);
+	
+	
+	if(bimaterial == 1){
+	  cout<<"______________________________________________________________________________"<<endl;
+	  cout<<"CASE OF BIMATERIAL CRACK  with lambda_up = "<<lambda_up<<" and lambda_down = "<<lambda_down<<endl;
+	  cout<<"______________________________________________________________________________"<<endl;
+	  std::vector<float> bi_lambda(ELAS.lambda().mf().nb_dof());
+	  std::vector<float> bi_mu(ELAS.lambda().mf().nb_dof());
+	  
+	  cout<<"ELAS.lambda().mf().nb_dof()==="<<ELAS.lambda().mf().nb_dof()<<endl;
+	  
+	  for (size_type ite = 0; ite < ELAS.lambda().mf().nb_dof();ite++) {
+	    if (ELAS.lambda().mf().point_of_dof(ite)[1] > 0){
+	      bi_lambda[ite] = lambda_up;
+	      bi_mu[ite] = mu_up;
+	    }
+	    else{
+	      bi_lambda[ite] = lambda_down;
+	      bi_mu[ite] = mu_down;
+	    }
+	  } 
+	  //cout<<"bi_lambda.size() = "<<bi_lambda.size()<<endl;
+	  // cout<<"ELAS.lambda().mf().nb_dof()==="<<ELAS.lambda().mf().nb_dof()<<endl;
+	  
+	  ELAS.lambda().set(bi_lambda);
+	  ELAS.mu().set(bi_mu);
+	}
+	
+	
+	// Defining the volumic source term.
+	
+	
+	plain_vector F(nb_dof_rhs * N);
+	for (size_type i = 0; i < nb_dof_rhs; ++i)
+	  gmm::copy(sol_f(mf_rhs.point_of_dof(i)),
+		    gmm::sub_vector(F, gmm::sub_interval(i*N, N)));
+	
+	// Volumic source term brick.
+	getfem::mdbrick_source_term<> VOL_F(ELAS, mf_rhs, F);
+	
+	// Defining the Neumann condition right hand side.
+	
+	// Neumann condition brick.
+	
+	getfem::mdbrick_abstract<> *pNEUMANN;
+	
+	gmm::clear(F);
+	for(size_type i = 0; i<F.size(); i=i+2) 
+	  {F[i] = F41; F[i+1] = F42;}
+	
+	getfem::mdbrick_source_term<> NEUMANN_down(VOL_F, mf_rhs, F,NEUMANN_BOUNDARY_NUM_down);
+	
+	gmm::clear(F);
+	for(size_type i = 0; i<F.size(); i=i+2) 
+	  {F[i] = F21; F[i+1] = F22;}
+	
+	getfem::mdbrick_source_term<> NEUMANN_right_up(NEUMANN_down, mf_rhs, F,NEUMANN_BOUNDARY_NUM_right);
+	
+	
+	gmm::clear(F);
+	for(size_type i = 0; i<F.size(); i=i+2) 
+	  {F[i] = F31; F[i+1] = F32;}
+	
+	getfem::mdbrick_source_term<> NEUMANN_right_down(NEUMANN_right_up, mf_rhs, F,NEUMANN_BOUNDARY_NUM_right+1);
+	
+	gmm::clear(F);
+	for(size_type i = 0; i<F.size(); i=i+2) 
+	  {F[i] = F11; F[i+1] = F12;}
+	
+	getfem::mdbrick_source_term<> NEUMANN_up(NEUMANN_right_down, mf_rhs, F,NEUMANN_BOUNDARY_NUM_up);
+	
+	if (all_dirichlet)
+	  pNEUMANN = & VOL_F; 
+	else 
+	  pNEUMANN = & NEUMANN_up; 
+	
+	//toto_solution toto(mf_rhs.linked_mesh()); toto.init();
+	//assert(toto.mf.nb_dof() == 1);
+	
+	// Dirichlet condition brick.
+	getfem::mdbrick_Dirichlet<> final_model(*pNEUMANN, DIRICHLET_BOUNDARY_NUM,
+						mf_mult);
+	if(all_dirichlet){
 #ifdef VALIDATE_XFEM
-    final_model.rhs().set(exact_sol.mf,exact_sol.U);
+	  final_model.rhs().set(exact_sol.mf,exact_sol.U);
 #endif
-  } else {
+	} else {
 #ifdef VALIDATE_XFEM
-  final_model.rhs().set(exact_sol.mf,0);
+	  final_model.rhs().set(exact_sol.mf,0);
 #endif
+	}
+	final_model.set_constraints_type(getfem::constraints_type(dir_with_mult));
+	
+	// Generic solve.
+	cout << "Total number of variables : " << final_model.nb_dof() << endl;
+	getfem::standard_model_state MS(final_model);
+	gmm::iteration iter(residual, 1, 40000);
+	
+	// getfem::standard_solve(MS, final_model, iter);
+	getfem::standard_solve(MS, final_model, iter, getfem::select_linear_solver(final_model, "superlu"));
+
+	// Solution extraction
+	gmm::copy(ELAS.get_solution(MS), U);
+	iteration = iter.converged();  
+	
+	// Adapted Refinement (suivant une erreur a posteriori)
+	if (adapted_refine && mode_counter == 1) {
+	  if (mesh.convex_index().size() < conv_max){
+	    plain_vector ERR(mesh.convex_index().last_true()+1);
+	    getfem::error_estimate(mim, mf_u(), U, ERR);
+	    
+	    cout << "max = " << gmm::vect_norminf(ERR) << endl;
+	    scalar_type threshold = 0.01, min_ = 1e18;
+	    conv_to_refine.clear();
+	    for (dal::bv_visitor i(mesh.convex_index()); !i.finished(); ++i) {
+	      if (ERR[i] > threshold && mesh.points()[mesh.ind_points_of_convex(i)[0]][0] >= -0.1) 
+		conv_to_refine.add(i);
+	      min_ = std::min(min_, ERR[i]);
+	    }
+	    cout << "min = " << min_ << endl;
+	    cout << "Refining " <<  conv_to_refine.card() << " convexes..."<< endl;  
+	    mesh.Bank_refine(conv_to_refine);
+
+	  }   
+	  else
+	    break;
+	  
+	}
+	if (mode_counter == 1) {
+	  mesh.write_to_file(datafilename + ".meshh");
+	  cout << "Refining process complete. The mesh contains now " <<  mesh.convex_index().size() << " convexes "<<endl;
+	  
+	  blocked_dof = mf_u().dof_on_set(5);
+	  mf_printed.set_finite_element(mesh.convex_index(),
+					getfem::fem_descriptor(FEM_DISC));
+	} else break;
+      } while (adapted_refine && conv_to_refine.size() > 0 );
+      
+      
+      plain_vector W1(mf_printed.nb_dof());
+      plain_vector W2(mf_printed.nb_dof());
+      cout << "nb of dof of printed_dof = " << mf_printed.nb_dof() << "." << endl;
+      cout << "Assembling final solution using the mode " << mode_counter << " solution..." << endl;
+      cout << "size of W1 and W2 = " << W1.size() << "." << endl;
+
+      switch (mode_counter){
+      
+      case 1:{
+	W12.resize(mf_printed.nb_dof()*2);
+	getfem::interpolation(mf_u(), mf_printed, U, W1);
+	size_type j = 0;
+	for(size_type i=0; i < W1.size(); i++){
+	  W12[j] = W1[i];
+	  j=j+2;  
+	}
+	cout << "W1[0]= " << W1[0] << "W1[1]= " << W1[1] << "W1[2]= " << W1[2] << "W1[3]= " << W1[3] << endl; 
+	cout << "W12[0]= " << W12[0] << " W12[1]= " << W12[1] << " W12[2]= " << W12[2] << " W12[3]= " << W12[3] << " W12[4]= " << W12[4] <<  " W12[5]= " << W12[5] <<  " W12[6]= " << W12[6]  << endl; 
+      } break;
+	
+      case 2:{
+	size_type j = 0;
+	getfem::interpolation(mf_u(), mf_printed, U, W2);
+	for(size_type i=0; i < W2.size(); i++){
+	  W12[j+1] = W2[i];
+	  j = j+2;
+	}
+      } break;
+      }
+      
+
+      mode_counter++;
+    } while(mode_counter < 3);
+    mf_printed.write_to_file(datafilename + "12.meshfem_", true);
+    mf_printed.set_qdim(4);  
+    mf_printed.write_to_file(datafilename + "12.meshfem", true);
+    gmm::vecsave(datafilename + "12.U", W12);
+
+    cout << "size of W12 = " << W12.size() << "." << endl;
+
+  } break; 
+    
   }
-  final_model.set_constraints_type(getfem::constraints_type(dir_with_mult));
-  
-  // Generic solve.
-  cout << "Total number of variables : " << final_model.nb_dof() << endl;
-  getfem::standard_model_state MS(final_model);
-  gmm::iteration iter(residual, 1, 40000);
-  getfem::standard_solve(MS, final_model, iter);
+  return (iteration);
 
-  // Solution extraction
-  gmm::copy(ELAS.get_solution(MS), U);
-
-  return (iter.converged());
 }
-  
+
+
 /**************************************************************************/
 /*  main program.                                                         */
 /**************************************************************************/
 
 int main(int argc, char *argv[]) {
 
-  DAL_SET_EXCEPTION_DEBUG; // Exceptions make a memory fault, to debug.
+  GMM_SET_EXCEPTION_DEBUG; // Exceptions make a memory fault, to debug.
   FE_ENABLE_EXCEPT;        // Enable floating point exception for Nan.
 
   //getfem::getfem_mesh_level_set_noisy();
@@ -663,8 +1015,8 @@ int main(int argc, char *argv[]) {
     p.init();
     p.mesh.write_to_file(p.datafilename + ".mesh");
     plain_vector U(p.mf_u().nb_dof());
-    if (!p.solve(U)) DAL_THROW(dal::failure_error,"Solve has failed");
-
+     if (!p.solve(U)) GMM_ASSERT1(false, "Solve has failed");
+   
     {
       getfem::mesh mcut;
       p.mls.global_cut_mesh(mcut);
@@ -759,8 +1111,8 @@ int main(int argc, char *argv[]) {
 	getfem::vtk_export exp(p.datafilename + ".vtk",
 			       p.PARAM.int_value("VTK_EXPORT")==1);
 	exp.exporting(mf_refined); 
-	exp.write_point_data(mf_refined_vm, DN, "error");
-	//exp.write_point_data(mf_refined_vm, VM, "von mises stress");
+	//exp.write_point_data(mf_refined_vm, DN, "error");
+	exp.write_point_data(mf_refined_vm, VM, "von mises stress");
 	exp.write_point_data(mf_refined, W, "elastostatic_displacement");
       
 #ifdef VALIDATE_XFEM
@@ -779,7 +1131,7 @@ int main(int argc, char *argv[]) {
 
 
 	getfem::interpolation_von_mises(mf_refined, mf_refined_vm, EXACT, VM_EXACT);
-	getfem::vtk_export exp2("crack_exact.vtk");
+	getfem::vtk_export exp2("bimaterial_crack_exact.vtk");
 	exp2.exporting(mf_refined);
 	exp2.write_point_data(mf_refined_vm, VM_EXACT, "exact von mises stress");
 	exp2.write_point_data(mf_refined, EXACT, "reference solution");
@@ -811,7 +1163,7 @@ int main(int argc, char *argv[]) {
     }
 
   }
-  DAL_STANDARD_CATCH_ERROR;
+  GMM_STANDARD_CATCH_ERROR;
 
   return 0; 
 }
