@@ -1,7 +1,7 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //========================================================================
 //
-// Copyright (C) 2002-2006 Michel Fourni, Julien Pommier, Yves Renard.
+// Copyright (C) 2002-2007 Michel Fourni, Julien Pommier,
 //
 // This file is a part of GETFEM++
 //
@@ -24,12 +24,12 @@
    @brief Fluid flow (Navier-Stokes) around an obstacle.
 */
 
-#include <getfem_assembling.h> /* import assembly methods (and norms comp.) */
-#include <getfem_export.h>   /* export functions (save solution in a file)  */
-#include <getfem_import.h>
-#include <getfem_regular_meshes.h>
-#include <getfem_model_solvers.h>
-#include <getfem_Navier_Stokes.h>
+#include "getfem/getfem_assembling.h" /* import assembly methods (and norms comp.) */
+#include "getfem/getfem_export.h"   /* export functions (save solution in a file)  */
+#include "getfem/getfem_import.h"
+#include "getfem/getfem_regular_meshes.h"
+#include "getfem/getfem_model_solvers.h"
+#include "getfem/getfem_Navier_Stokes.h"
 #include "icare.h"
 
 /* some Getfem++ types that we will be using */
@@ -66,6 +66,7 @@ struct navier_stokes_problem {
   getfem::mesh_fem mf_u;     /* main mesh_fem, for the velocity              */
   getfem::mesh_fem mf_p;     /* mesh_fem for the pressure                    */
   getfem::mesh_fem mf_rhs;   /* mesh_fem for the right hand side (f(x),..)   */
+  getfem::mesh_fem mf_mult;  /* mesh_fem for Dirichlet condition.            */
   scalar_type Re;            /* Reynolds number */
   scalar_type nu;            /* 1/Re */
   scalar_type dt, T, dt_export;
@@ -80,18 +81,12 @@ struct navier_stokes_problem {
   scalar_type t_export;
   void do_export(scalar_type t);
 
-  typedef enum {
-    METHOD_SPLITTING_STOKES = 0,      /* cf. works of Michel Fournie. */
-    METHOD_SPLITTING = 1,             /* cf. works of Michel Fournie. */
-    METHOD_FULLY_CONSERVATIVE = 2, 
-    METHOD_PREDICTION_CORRECTION = 3  /* cf. works of Marianna Braza. */
-  } option_t;
-  option_t option;
+  int option;
 
   std::auto_ptr<problem_definition> pdef;
 
   std::string datafilename;
-  ftool::md_param PARAM;
+  bgeot::md_param PARAM;
 
   plain_vector Un1, Un0, Pn1, Pn0; /* U_{n+1}, U_{n}, P_{n+1} and P_{n} */
 
@@ -102,9 +97,10 @@ struct navier_stokes_problem {
   void solve_METHOD_SPLITTING(bool);
   void solve_FULLY_CONSERVATIVE();
   void solve_PREDICTION_CORRECTION();
+  void solve_PREDICTION_CORRECTION2();
   void init(void);
   navier_stokes_problem(void) : mim(mesh), mf_u(mesh), mf_p(mesh),
-				mf_rhs(mesh) {}
+				mf_rhs(mesh), mf_mult(mesh) {}
 };
 
 struct problem_definition {
@@ -233,10 +229,13 @@ struct problem_rotating_cylinder : public problem_definition {
     getfem::mesh_region r; 
     getfem::outer_faces_of_mesh(p.mesh, r);
     for (getfem::mr_visitor i(r); !i.finished(); ++i) {
-      base_node G = dal::mean_value(p.mesh.points_of_face_of_convex(i.cv(),i.f()));
-      if (gmm::abs(G[0] - p.BBmax[0]) < 1e-7)
+      base_node G = gmm::mean_value(p.mesh.points_of_face_of_convex(i.cv(),i.f()));
+      /*if (gmm::abs(G[0] - p.BBmax[0]) < 1e-7)
 	p.mesh.region(NONREFLECTIVE_BOUNDARY_NUM).add(i.cv(),i.f());
-      else 
+      else if (gmm::abs(G[1] - p.BBmax[1]) < 1e-7
+	       || gmm::abs(G[1] - p.BBmin[1]) < 1e-7)
+	p.mesh.region(NORMAL_PART_DIRICHLET_BOUNDARY_NUM).add(i.cv(),i.f());
+	else */
 	p.mesh.region(DIRICHLET_BOUNDARY_NUM).add(i.cv(),i.f());
     }
   }
@@ -251,8 +250,8 @@ struct problem_rotating_cylinder : public problem_definition {
     if (gmm::abs(x-p.BBmax[0]) < 1e-7) on_cylinder = false;
     if (gmm::abs(y-p.BBmax[1]) < 1e-7) on_cylinder = false;
     if (!on_cylinder) {
-      if (!(gmm::abs(y- p.BBmin[1]) < 1e-7 || gmm::abs(y - p.BBmax[1])< 1e-7))
-	r[0] = 1;
+      //if (!(gmm::abs(y- p.BBmin[1]) < 1e-7 || gmm::abs(y - p.BBmax[1])< 1e-7))
+      r[0] = 1;
     } else {
       r[0] = -2.*alpha*y; /* HYPOTHESIS: cylinder centered at (0,0) */
       r[1] = 2.*alpha*x;
@@ -303,21 +302,22 @@ void navier_stokes_problem::init(void) {
   mesh.bounding_box(BBmin, BBmax);
   cout << "mesh bounding box: " << BBmin << " ... " << BBmax << "\n";
   datafilename = PARAM.string_value("ROOTFILENAME","Data files base name.");
-  residual = PARAM.real_value("RESIDUAL"); if (residual == 0.) residual = 1e-10;
+  residual = PARAM.real_value("RESIDUAL"); 
+  if (residual == 0.) residual = 1e-10;
 
   nu = PARAM.real_value("NU", "Viscosity");
   dt = PARAM.real_value("DT", "Time step");
   T = PARAM.real_value("T", "Final time");
   dt_export = PARAM.real_value("DT_EXPORT", "Time step for export");
   noisy = PARAM.int_value("NOISY", "");
-  option = option_t(PARAM.int_value("OPTION", "option"));
+  option = PARAM.int_value("OPTION", "option");
 
   int prob = PARAM.int_value("PROBLEM", "the problem");
   switch (prob) {
     case 1: pdef.reset(new problem_definition_Stokes_analytic); break;
     case 2: pdef.reset(new problem_definition_Green_Taylor_analytic); break;
     case 3: pdef.reset(new problem_rotating_cylinder(PARAM.real_value("CYL_ROT_SPEED"))); break;
-    default: DAL_THROW(dal::failure_error, "wrong PROBLEM value");
+    default: GMM_ASSERT1(false, "wrong PROBLEM value");
   }
 
   export_to_opendx = PARAM.int_value("DX_EXPORT", "");
@@ -327,8 +327,7 @@ void navier_stokes_problem::init(void) {
   mf_u.set_qdim(N);
 
   /* set the finite element on the mf_u */
-  getfem::pfem pf_u = 
-    getfem::fem_descriptor(FEM_TYPE);
+  getfem::pfem pf_u = getfem::fem_descriptor(FEM_TYPE);
   getfem::pintegration_method ppi = 
     getfem::int_method_descriptor(INTEGRATION); 
 
@@ -341,17 +340,26 @@ void navier_stokes_problem::init(void) {
      not used in the .param file */
   std::string data_fem_name = PARAM.string_value("DATA_FEM_TYPE");
   if (data_fem_name.size() == 0) {
-    if (!pf_u->is_lagrange()) {
-      DAL_THROW(dal::failure_error, "You are using a non-lagrange FEM "
-		<< data_fem_name << ". In that case you need to set "
+    GMM_ASSERT1(pf_u->is_lagrange(), "You are using a non-lagrange FEM "
+		<< FEM_TYPE << ". In that case you need to set "
 		<< "DATA_FEM_TYPE in the .param file");
-    }
     mf_rhs.set_finite_element(mesh.convex_index(), pf_u);
   } else {
     mf_rhs.set_finite_element(mesh.convex_index(), 
 			      getfem::fem_descriptor(data_fem_name));
   }
 
+  std::string mult_fem_name = PARAM.string_value("MULTIPLIER_FEM_TYPE");
+  if (mult_fem_name.size() == 0) {
+    GMM_ASSERT1(pf_u->is_lagrange(), "You are using a non-lagrange FEM "
+		<< FEM_TYPE << ". In that case you need to set "
+		<< "MULTIPLIER_FEM_TYPE in the .param file");
+    mf_mult.set_finite_element(mesh.convex_index(), pf_u);
+  } else {
+    mf_mult.set_finite_element(mesh.convex_index(), 
+			      getfem::fem_descriptor(mult_fem_name));
+  }
+
   /* set boundary conditions */
   cout << "Choosing boundaries\n";
   pdef->choose_boundaries(*this);
@@ -366,11 +374,12 @@ void navier_stokes_problem::solve() {
   gmm::resize(Pn0, mf_p.nb_dof());
   gmm::resize(Pn1, mf_p.nb_dof());
   switch (option) {
-    case METHOD_SPLITTING_STOKES: solve_METHOD_SPLITTING(true); break;
-    case METHOD_SPLITTING: solve_METHOD_SPLITTING(false); break;
-    case METHOD_FULLY_CONSERVATIVE: solve_FULLY_CONSERVATIVE(); break;
-    case METHOD_PREDICTION_CORRECTION: solve_PREDICTION_CORRECTION(); break;
-    default: DAL_THROW(dal::failure_error, "unknown method");
+  case 0 : solve_METHOD_SPLITTING(true); break;
+  case 1 : solve_METHOD_SPLITTING(false); break;
+  case 2 : solve_FULLY_CONSERVATIVE(); break;
+  case 3 : solve_PREDICTION_CORRECTION(); break;
+  case 4 : solve_PREDICTION_CORRECTION2(); break;
+  default: GMM_ASSERT1(false, "unknown method");
   }
 }
 
@@ -410,9 +419,8 @@ void navier_stokes_problem::solve_METHOD_SPLITTING(bool stokes_only) {
     velocity_dirnp(velocity_dir, NORMAL_PART_DIRICHLET_BOUNDARY_NUM, mf_rhs);
 
   // Non-reflective condition brick
-  getfem::mdbrick_NS_nonref1<> nonreflective(velocity_dirnp, mf_u, 
-					     NONREFLECTIVE_BOUNDARY_NUM,
-					     dt, nu);
+  getfem::mdbrick_NS_nonref1<> nonreflective(velocity_dirnp,
+					     NONREFLECTIVE_BOUNDARY_NUM, dt);
  
   // Dynamic brick.
   getfem::mdbrick_dynamic<> velocity_dyn(nonreflective, 1.);
@@ -425,13 +433,13 @@ void navier_stokes_problem::solve_METHOD_SPLITTING(bool stokes_only) {
   getfem::mdbrick_mass_matrix<> mixed(mim, mf_u, 1./dt);
   
   // Pressure term
-  getfem::mdbrick_linear_incomp<>mixed_p(mixed, mf_p);
+  getfem::mdbrick_linear_incomp<> mixed_p(mixed, mf_p);
 
   // Condition on the pressure
   sparse_matrix G(1, mf_p.nb_dof());
   G(0,0) = 1.;
   plain_vector gr(1);
-  getfem::mdbrick_constraint<> set_pressure(mixed_p);
+  getfem::mdbrick_constraint<> set_pressure(mixed_p, 1);
   set_pressure.set_constraints(G, gr);
   set_pressure.set_constraints_type(getfem::AUGMENTED_CONSTRAINTS);
     
@@ -517,7 +525,7 @@ void navier_stokes_problem::solve_FULLY_CONSERVATIVE() {
 
   // Non-reflective condition brick
   getfem::mdbrick_NS_nonref1<>
-    nonreflective(velocity_dirnp, mf_u, NONREFLECTIVE_BOUNDARY_NUM, dt, nu);
+    nonreflective(velocity_dirnp, NONREFLECTIVE_BOUNDARY_NUM, dt);
 
   // Dynamic brick.
   getfem::mdbrick_dynamic<> velocity_dyn(nonreflective, 1.);
@@ -559,7 +567,7 @@ void navier_stokes_problem::solve_FULLY_CONSERVATIVE() {
   }
 }
 
-
+/************************************************************/
 void navier_stokes_problem::solve_PREDICTION_CORRECTION() {
   // Velocity brick.  
   getfem::mdbrick_generic_elliptic<> velocity(mim, mf_u, nu);
@@ -578,8 +586,9 @@ void navier_stokes_problem::solve_PREDICTION_CORRECTION() {
     velocity_dirnp(velocity_dir, NORMAL_PART_DIRICHLET_BOUNDARY_NUM, mf_rhs);
   
   // Non-reflective condition brick
-  getfem::mdbrick_NS_nonref1<> nonreflective(velocity_dirnp, mf_u, 
-					      NONREFLECTIVE_BOUNDARY_NUM, dt, nu);
+  getfem::mdbrick_NS_nonref1<> nonreflective( velocity_dirnp, 
+					      NONREFLECTIVE_BOUNDARY_NUM, dt);
+  
 
   // Dynamic brick.
     getfem::mdbrick_dynamic<> velocity_dyn(nonreflective, 1.);
@@ -616,7 +625,6 @@ void navier_stokes_problem::solve_PREDICTION_CORRECTION() {
   do_export(0);
   for (scalar_type t = dt; t <= T; t += dt) {
 
-
     velocity_nonlin.set_U0(Un0);
     nonreflective.set_Un(Un0);
     pdef->source_term(*this, t, F);
@@ -633,6 +641,9 @@ void navier_stokes_problem::solve_PREDICTION_CORRECTION() {
     gmm::copy(velocity.get_solution(MSL), USTAR);
     gmm::mult(B, USTAR, Pn1);
  
+    cout << "PN1 : " << gmm::vect_norm2(Pn1) << endl;
+    cout << "U1 - USTAR = " << gmm::vect_dist2(Un0, USTAR);
+
     poisson_source.set_auxF(Pn1);
     iter.init();
     getfem::standard_solve(MSM, poisson_source, iter);
@@ -647,6 +658,289 @@ void navier_stokes_problem::solve_PREDICTION_CORRECTION() {
     gmm::add(gmm::scaled(poisson.get_solution(MSM), 1./dt), Pn0, Pn1);
     
     pdef->validate_solution(*this, t);
+    cout << "U1 - Un1 = " << gmm::vect_dist2(Un1, Un0);
+    
+    gmm::copy(Un1, Un0); gmm::copy(Pn1, Pn0);
+    do_export(t);
+
+ }
+}
+
+
+/************************************************************/
+void navier_stokes_problem::solve_PREDICTION_CORRECTION2() {
+
+  size_type nbdof_u = mf_u.nb_dof(), nbdof_p = mf_p.nb_dof();
+  size_type nbdof_rhs = mf_rhs.nb_dof();
+  getfem::mesh_region mpirg = mf_u.linked_mesh().get_mpi_region();
+  gmm::sub_interval I1(0, nbdof_u);
+
+  cout << "nbdof rhs = " << nbdof_rhs << endl;
+
+  // Discretization of laplace operator for u
+  sparse_matrix K1(nbdof_u, nbdof_u);
+  asm_stiffness_matrix_for_homogeneous_laplacian_componentwise
+    (K1, mim, mf_u, mpirg);
+  gmm::scale(K1, nu);
+  
+  // Mass Matrix
+  sparse_matrix M(nbdof_u, nbdof_u);
+  asm_mass_matrix(M, mim, mf_u, mpirg);
+  
+
+
+  // Matrix p div u
+  sparse_matrix B(nbdof_p, nbdof_u);
+  asm_stokes_B(B, mim, mf_u, mf_p, mpirg);
+  
+  mf_mult.set_qdim(N);
+  dal::bit_vector dofon_Dirichlet = mf_mult.dof_on_region(DIRICHLET_BOUNDARY_NUM);
+  dal::bit_vector dofon_nonref =mf_mult.dof_on_region(NONREFLECTIVE_BOUNDARY_NUM);
+  dofon_Dirichlet.setminus(dofon_nonref);
+
+  // Normal part Dirichlet condition
+  mf_mult.set_qdim(1);
+  dal::bit_vector dofon_NDirichlet
+    = mf_mult.dof_on_region(NORMAL_PART_DIRICHLET_BOUNDARY_NUM);
+  std::vector<size_type> ind_ct_ndir;
+  for (dal::bv_visitor i(dofon_NDirichlet); !i.finished(); ++i) {
+    if (dofon_Dirichlet.is_in(i*N) || dofon_nonref.is_in(i*N)) 
+      dofon_NDirichlet.sup(i);  // Suppress i because it is on the
+    else                        // Dirichlet or non reflective boundary.
+      ind_ct_ndir.push_back(i);
+  }
+  size_type nbdof_NDir = dofon_NDirichlet.card();
+  gmm::sub_index SUB_CT_NDIR(ind_ct_ndir);
+  gmm::sub_interval I2(nbdof_u, nbdof_NDir);
+  getfem::mesh_region mpindirrg
+    =mf_u.linked_mesh().get_mpi_sub_region(NORMAL_PART_DIRICHLET_BOUNDARY_NUM);
+
+  sparse_matrix HND(nbdof_NDir, nbdof_u);
+  {
+    sparse_matrix A(mf_mult.nb_dof(), nbdof_u); 
+    getfem::generic_assembly assem;
+    assem.set("M(#2,#1)+=comp(Base(#2).vBase(#1).Normal())(:,:,i,i);");
+    assem.push_mi(mim); assem.push_mf(mf_u); assem.push_mf(mf_mult);
+    assem.push_mat(A); assem.assembly(mpindirrg);
+    gmm::copy(gmm::sub_matrix(A, SUB_CT_NDIR, I1), HND);
+  }
+  cout << "Nb of Normal par Dirichlet constraints : " << nbdof_NDir << endl;
+
+  // Dirichlet condition
+  mf_mult.set_qdim(N);
+  size_type nbdof_Dir = dofon_Dirichlet.card();
+  std::vector<size_type> ind_ct_dir;
+  for (dal::bv_visitor i(dofon_Dirichlet); !i.finished(); ++i) 
+    ind_ct_dir.push_back(i);
+  gmm::sub_index SUB_CT_DIR(ind_ct_dir);
+  gmm::sub_interval I3(nbdof_u+nbdof_NDir, nbdof_Dir);
+  getfem::mesh_region mpidirrg
+    = mf_u.linked_mesh().get_mpi_sub_region(DIRICHLET_BOUNDARY_NUM);
+
+  sparse_matrix HD(nbdof_Dir, nbdof_u);
+  {
+    sparse_matrix A(mf_mult.nb_dof(), nbdof_u); 
+    getfem::generic_assembly assem;
+    assem.set("M(#2,#1)+=comp(vBase(#2).vBase(#1))(:,i,:,i);");
+    assem.push_mi(mim); assem.push_mf(mf_u); assem.push_mf(mf_mult);
+    assem.push_mat(A); assem.assembly(mpidirrg);
+    gmm::copy(gmm::sub_matrix(A, SUB_CT_DIR, I1), HD);
+  }
+  cout << "Nb of Dirichlet constraints : " << nbdof_Dir << endl;
+
+
+  // Non reflective condition
+  size_type nbdof_nonref = dofon_nonref.card();
+  std::vector<size_type> ind_ct_nonref;
+  for (dal::bv_visitor i(dofon_nonref); !i.finished(); ++i) 
+    ind_ct_nonref.push_back(i);
+  gmm::sub_index SUB_CT_NONREF(ind_ct_nonref);
+  gmm::sub_interval I4(nbdof_u+nbdof_NDir+nbdof_Dir, nbdof_nonref);
+  getfem::mesh_region mpinonrefrg
+    = mf_u.linked_mesh().get_mpi_sub_region(NONREFLECTIVE_BOUNDARY_NUM);
+
+  sparse_matrix HNR(nbdof_nonref, nbdof_u);
+  {
+    sparse_matrix A(mf_mult.nb_dof(), nbdof_u); 
+    getfem::generic_assembly assem;
+    assem.set("M(#2,#1)+=comp(vBase(#2).vBase(#1))(:,i,:,i);");
+    assem.push_mi(mim); assem.push_mf(mf_u); assem.push_mf(mf_mult);
+    assem.push_mat(A); assem.assembly(mpinonrefrg);
+    gmm::copy(gmm::sub_matrix(A, SUB_CT_NONREF, I1), HNR);
+
+  
+  }
+  cout << "Nb on Non reflective condition: " << nbdof_nonref << endl;
+
+
+  // Discretization of laplace operator for p
+  sparse_matrix K2(nbdof_p+1, nbdof_p+1);
+  gmm::sub_interval IP(0, nbdof_p);
+  asm_stiffness_matrix_for_homogeneous_laplacian(gmm::sub_matrix(K2, IP),
+						 mim, mf_p, mpirg);
+  K2(nbdof_p, 0) = K2(0, nbdof_p) = 1.0; // set the first pressure dof to 0
+  
+
+  // 
+  // dynamic problem
+  //
+  plain_vector DF(nbdof_u), F(nbdof_rhs), USTAR(nbdof_u), USTARbis(nbdof_u);
+  plain_vector Phi(nbdof_p);
+
+  pdef->initial_condition_u(*this, Un0);
+  pdef->initial_condition_p(*this, Pn0);
+
+  //gmm::clear(Un0); gmm::clear(Pn0);
+  gmm::vecsave("Un0", Un0);
+  gmm::vecsave("Pn0", Pn0);
+  
+  do_export(0);
+  for (scalar_type t = dt; t <= T; t += dt) {
+
+    //
+    // Assembly of the first linear system
+    //
+    size_type sizelsystem = nbdof_u + nbdof_NDir + nbdof_Dir + nbdof_nonref;
+    sparse_matrix A1(sizelsystem, sizelsystem);
+   
+    sparse_matrix A2(sizelsystem, sizelsystem);  // sera utilise pour reappliquer la CNR
+    
+    plain_vector Y(sizelsystem), YY(nbdof_u);
+    // laplace operator
+    gmm::copy(K1, gmm::sub_matrix(A1, I1));
+
+    // Nonlinear part
+    getfem::asm_NS_uuT(gmm::sub_matrix(A1, I1), mim, mf_u, Un0, mpirg);
+    // Dynamic part
+    gmm::add(gmm::scaled(M, 1./dt), gmm::sub_matrix(A1, I1));
+        
+     gmm::add(M, gmm::sub_matrix(A2, I1));
+
+    
+    gmm::mult(M, gmm::scaled(Un0, 1./dt), gmm::sub_vector(Y, I1));
+
+    //    plain_vector subY1(nbdof_u);
+    // gmm::mult(M, gmm::scaled(Un0, 1./dt), subY1);
+    /*
+    plain_vector subY1(nbdof_u);
+    gmm::mult(M, gmm::scaled(Un0, 1./dt), subY1);
+
+    // Volumic source term
+    pdef->source_term(*this, t, F);
+    getfem::asm_source_term(subY1, mim, mf_u, mf_rhs, F,
+			    mpirg);
+    */
+    // Normal Dirichlet condition
+    gmm::copy(HND, gmm::sub_matrix(A1, I2, I1));
+    gmm::copy(gmm::transposed(HND), gmm::sub_matrix(A1, I1, I2));
+    
+    gmm::copy(HND, gmm::sub_matrix(A2, I2, I1));
+    gmm::copy(gmm::transposed(HND), gmm::sub_matrix(A2, I1, I2));
+    
+    // Dirichlet condition
+    gmm::copy(HD, gmm::sub_matrix(A1, I3, I1));
+    gmm::copy(gmm::transposed(HD), gmm::sub_matrix(A1, I1, I3));
+    
+    gmm::copy(HD, gmm::sub_matrix(A2, I3, I1));
+    gmm::copy(gmm::transposed(HD), gmm::sub_matrix(A2, I1, I3));
+    
+    gmm::resize(F, N * nbdof_rhs);
+    pdef->dirichlet_condition(*this, t, F);
+    {
+      plain_vector VV(mf_mult.nb_dof());
+      getfem::asm_source_term(VV, mim, mf_mult, mf_rhs, F, mpidirrg);
+      gmm::copy(gmm::sub_vector(VV, SUB_CT_DIR), gmm::sub_vector(Y, I3));
+    }
+    // Non reflective condition
+    gmm::copy(HNR, gmm::sub_matrix(A1, I4, I1));
+    gmm::copy(gmm::transposed(HNR), gmm::sub_matrix(A1, I1, I4));
+    
+    gmm::copy(HNR, gmm::sub_matrix(A2, I4, I1));
+    gmm::copy(gmm::transposed(HNR), gmm::sub_matrix(A2, I1, I4));
+    {
+      plain_vector VV(mf_mult.nb_dof());
+    /*  if (t < 0.2)
+	getfem::asm_source_term(VV, mim, mf_mult, mf_rhs, F, mpinonrefrg);
+      else {
+	*/
+	//getfem::asm_source_term(VV, mim, mf_mult, mf_rhs, Un0, mpinonrefrg);
+	
+      //}
+	getfem::generic_assembly assem;
+    // construction du terme de droite dans [M]*Unp1=F
+    
+    // mise en place de Un + Un.N*(dUn/dn).N
+
+    std::stringstream ss;
+    ss << "u=data$1(#1); "
+      "V(#1)+="
+       << -dt << "*"
+      " comp(vBase(#1).Normal().vGrad(#1).Normal().vBase(#2))"
+      "(l,i,i,m,j,k,j,:,k).u(l).u(m)+"
+      "comp(vBase(#1).vBase(#2))(j,i,:,i).u(j)";
+ 
+    assem.set(ss.str());
+    assem.push_mi(mim);
+    assem.push_mf(mf_u);
+    assem.push_mf(mf_mult);  
+    assem.push_data(Un0);
+    assem.push_vec(VV);
+    assem.assembly(mpinonrefrg);
+      gmm::copy(gmm::sub_vector(VV, SUB_CT_NONREF), gmm::sub_vector(Y, I4));
+    }
+
+    // pressure part
+    gmm::mult(gmm::transposed(B), gmm::scaled(Pn0, -1.0), YY);
+    gmm::add(YY, gmm::sub_vector(Y, I1));
+    
+    //
+    // Solving the first linear system
+    //
+    {
+      double rcond;
+      plain_vector X(sizelsystem);
+      SuperLU_solve(A1, X, Y, rcond);
+      // if (noisy) cout << "condition number: " << 1.0/rcond << endl;
+      gmm::copy(gmm::sub_vector(X, I1), USTAR);
+      
+      gmm::HarwellBoeing_IO::write("A1.hb", A1);
+      gmm::vecsave("Y", Y);
+      gmm::vecsave("USTAR", USTAR); exit(1);
+    }
+
+    cout << "U* - Un0 = " << gmm::vect_dist2(USTAR, Un0) << endl;
+
+    //
+    // Solving the second linear system
+    //
+    {
+      double rcond;
+      plain_vector X(nbdof_p+1), Z(nbdof_p+1);
+      gmm::mult(B, USTAR, gmm::sub_vector(Z, IP));
+      SuperLU_solve(K2, X, Z, rcond);
+      // if (noisy) cout << "condition number: " << 1.0/rcond << endl;
+      gmm::copy(gmm::sub_vector(X, IP), Phi);
+    }
+
+    gmm::mult(M, USTAR, USTARbis);
+    gmm::mult(gmm::transposed(B), gmm::scaled(Phi, -1.), USTARbis, USTARbis);
+    gmm::copy(USTARbis, gmm::sub_vector(Y, I1));
+    //gmm::copy(M, gmm::sub_matrix(A1, I1));
+    
+    {
+      double rcond;
+      plain_vector X(sizelsystem);
+      SuperLU_solve(A2, X, Y, rcond);     // A2 contient la matrice de masse et les blocs des CL
+      gmm::copy(gmm::sub_vector(X, I1), Un1);
+
+    }
+   
+
+    // gmm::add(USTAR, Un1);
+    gmm::add(gmm::scaled(Phi, 1./dt), Pn0, Pn1);
+    
+    pdef->validate_solution(*this, t);
+    cout << "Un1 - Un0 = " << gmm::vect_dist2(Un1, Un0) << ", t=" << t << ", T=" << T << endl;
     
     gmm::copy(Un1, Un0); gmm::copy(Pn1, Pn0);
     do_export(t);
@@ -654,10 +948,12 @@ void navier_stokes_problem::solve_PREDICTION_CORRECTION() {
 }
 
 
+
 void navier_stokes_problem::do_export(scalar_type t) {
   if (!export_to_opendx) return;
   if (first_export) {
     mf_u.write_to_file("icare.mf_u", true);
+    mf_p.write_to_file("icare.mf_p", true);
 
     exp.reset(new getfem::dx_export(datafilename + ".dx", false));
     if (N <= 2)
@@ -668,19 +964,38 @@ void navier_stokes_problem::do_export(scalar_type t) {
     exp->exporting_mesh_edges();
     t_export = 0;
     first_export = false;
-  } else if (t >= t_export-dt/20.0) {
-    t_export += dt_export;
   }
-  
-  static int cnt = 0;
-  char s[128]; sprintf(s, "icare.U%d", cnt++);
-  gmm::vecsave(s, Un0);
+  if (t >= t_export-dt/20.0) {
+    t_export += dt_export;
+    
+    static int cnt = 0;
+    char s[128]; sprintf(s, "icare.U%d", cnt++);
+    gmm::vecsave(s, Un0);
+    
+    exp->write_point_data(mf_u, Un0);
+    exp->serie_add_object("velocity");
+    cout << "Saving Pressure, |p| = " << gmm::vect_norm2(Pn1) << "\n";
+    exp->write_point_data(mf_p, Pn1);
+    exp->serie_add_object("pressure");
 
-  exp->write_point_data(mf_u, Un0);
-  exp->serie_add_object("velocity");
-  cout << "Saving Pressure, |p| = " << gmm::vect_norm2(Pn1) << "\n";
-  exp->write_point_data(mf_p, Pn1);
-  exp->serie_add_object("pressure");
+        
+    static int cntp=0;
+    char sp[128]; sprintf(sp, "icare.P%d", cntp++);
+    gmm::vecsave(sp, Pn0);
+    
+    if (N == 2) {
+      plain_vector DU(mf_rhs.nb_dof() * N * N);
+      plain_vector Rot(mf_rhs.nb_dof());
+      compute_gradient(mf_u, mf_rhs, Un0, DU);
+      for (unsigned i=0; i < mf_rhs.nb_dof(); ++i) {
+	Rot[i] = DU[i*N*N + 3] - DU[i*N*N + 2];
+	if ((Rot[i]*Rot[i])<=1.5){Rot[i]=0;}
+      }
+      cout << "Saving Rot, |rot| = " << gmm::vect_norm2(Rot) << "\n";
+      exp->write_point_data(mf_rhs, Rot);
+      exp->serie_add_object("rot");
+    }
+  }
 }
 
 /**************************************************************************/
@@ -689,7 +1004,7 @@ void navier_stokes_problem::do_export(scalar_type t) {
 
 int main(int argc, char *argv[]) {
 
-  DAL_SET_EXCEPTION_DEBUG; // Exceptions make a memory fault, to debug.
+  GMM_SET_EXCEPTION_DEBUG; // Exceptions make a memory fault, to debug.
   FE_ENABLE_EXCEPT;        // Enable floating point exception for Nan.
 
   try {    
@@ -699,7 +1014,7 @@ int main(int argc, char *argv[]) {
     p.mesh.write_to_file(p.datafilename + ".mesh");
     p.solve();
   }
-  DAL_STANDARD_CATCH_ERROR;
+  GMM_STANDARD_CATCH_ERROR;
 
   return 0; 
 }
