@@ -132,9 +132,10 @@ namespace dal {
       cerr << "Problem adding dependency between " << o1 << " of type "
 	   << typeid(*o1).name() << " and " << o2 << " of type "
 	   << typeid(*o2).name() << ". ";
-      if (it1 == stored_objects.end()) cerr << "First object does non exist.";
-      if (it2 == stored_objects.end()) cerr << "Second object does non exist.";
+      if (it1 == stored_objects.end()) cerr << "First object does not exist.";
+      if (it2 == stored_objects.end()) cerr << "Second object does not exist.";
       cerr << endl;
+      assert(false);
       GMM_ASSERT1(false, "Add_dependency : Inexistent object");
     }
   }
@@ -229,7 +230,7 @@ namespace dal {
 	  stored_object_tab::iterator itod=iterator_of_object(*itd);
 	  if (itod != stored_objects.end()) {
 	    GMM_ASSERT1(itod->second.perm != PERMANENT_STATIC_OBJECT,
-			"Trying to delete a permanent object");
+			"Trying to delete a permanent object " << *itd);
 	    if (itod->second.valid) {
 	      itod->second.valid = false;
 	      to_delete.push_back(itod->second.p);
