@@ -156,7 +156,8 @@ namespace getfem {
     std::vector<base_tensor> val_e(pfems.size());
     for (size_type k = 0; k < pfems.size(); ++k) {
       if (c0.have_pfp()) {
-	c0.set_pfp(fem_precomp(pfems[k], &c0.pfp()->get_point_tab()));
+	c0.set_pfp(fem_precomp(pfems[k], &c0.pfp()->get_point_tab(),
+			       c0.pfp()));
       } else { c0.set_pf(pfems[k]); }
       c0.base_value(val_e[k]);
     }
@@ -187,7 +188,8 @@ namespace getfem {
     std::vector<base_tensor> grad_e(pfems.size());
     for (size_type k = 0; k < pfems.size(); ++k) {
       if (c0.have_pfp()) {
-	c0.set_pfp(fem_precomp(pfems[k], &c0.pfp()->get_point_tab()));
+	c0.set_pfp(fem_precomp(pfems[k], &c0.pfp()->get_point_tab(),
+			       c0.pfp()));
       } else { c0.set_pf(pfems[k]); }
       c0.grad_base_value(grad_e[k]);
     }
@@ -220,7 +222,8 @@ namespace getfem {
     std::vector<base_tensor> hess_e(pfems.size());
     for (size_type k = 0; k < pfems.size(); ++k) {
       if (c0.have_pfp()) {
-	c0.set_pfp(fem_precomp(pfems[k], &c0.pfp()->get_point_tab()));
+	c0.set_pfp(fem_precomp(pfems[k], &c0.pfp()->get_point_tab(),
+			       c0.pfp()));
       } else { c0.set_pf(pfems[k]); }
       c0.hess_base_value(hess_e[k]);
     }
