@@ -23,6 +23,7 @@
 #include "getfem/bgeot_poly_composite.h"
 #include "getfem/bgeot_comma_init.h"
 #include "getfem/getfem_export.h"
+#include "getfem/bgeot_node_tab.h"
 
 
 using getfem::size_type;
@@ -327,32 +328,232 @@ void test_convex_ref() {
   }
 }
 
-int main(void) {
-  GMM_SET_EXCEPTION_DEBUG; // Exceptions make a memory fault, to debug.
-  FE_ENABLE_EXCEPT;        // Enable floating point exception for Nan.
+
+
+struct basic_mesh_point_comparator2
+  : public std::binary_function<base_node, base_node, int> {
+  double eps;
+  std::vector<double> v;
   
-  try {
-    getfem::mesh m1;
-    test_convex_ref();
-    test_convex_simplif();
-    test_mesh(m1);
-    test_convex_simplif();
-    test_convex_quality(0.2,0);
-    test_convex_quality(-0.2,0);
-    test_convex_quality(-0.01,-0.2);
-    test_region();
-
-    for (size_type d = 1; d <= 4 /* 6 */; ++d)
-      test_mesh_matching(d);
-
-    test_refinable(2, 1);
-    test_refinable(2, 2);
-    test_refinable(2, 3);
-    test_refinable(3, 1);
-    test_refinable(3, 2);
-    test_refinable(3, 3);
+  int operator()(const base_node &x, const base_node &y) const {
+    double a = gmm::vect_sp(x, v), b =  gmm::vect_sp(y, v);
+    if (a < b - eps) return -1; else if (a > b + eps) return +1; else return 0;
   }
-  GMM_STANDARD_CATCH_ERROR;
+  
+  basic_mesh_point_comparator2(unsigned dim_ = 3, double e = double(10000)
+			       *gmm::default_tol(double()))
+    : eps(e), v(dim_) {
+    gmm::fill_random(v);
+    gmm::scale(v, 1.0/gmm::vect_norm2(v));
+  }
+};
+
+
+void test_mesh_building(int dim, int Nsubdiv) {
+
+  double exectime = gmm::uclock_sec();
+
+  std::vector<size_type> nsubdiv(dim, Nsubdiv);
+  getfem::mesh m;
+  bgeot::pgeometric_trans pgt = bgeot::simplex_geotrans(dim, 1);
+  getfem::regular_unit_mesh(m, nsubdiv, pgt,false);
+
+  cout << "Time to create mesh : "
+       << gmm::uclock_sec() - exectime << endl;
+
+//   // Test of a simple sort on one "random" component
+//   exectime = gmm::uclock_sec();
+//   typedef basic_mesh_point_comparator2 pt_comp2;
+//   typedef dal::dynamic_tree_sorted<base_node, pt_comp2> PT_TAB2;
+  
+//   PT_TAB2 pttab2 = PT_TAB2(pt_comp2(dim));
+
+//   for (dal::bv_visitor i(m.convex_index()); !i.finished(); ++i) {
+//     for (int j = 0; j <= dim; ++j)
+//       pttab2.add_norepeat(m.points_of_convex(i)[j]);
+//   }
+//   cout << "Time to insert points in structure with random component: "
+//        << gmm::uclock_sec() - exectime << endl;
+//   GMM_ASSERT1(pttab2.card() == m.nb_points(),
+// 	      "Problem in identifying points " << pttab2.card()
+// 	      << " : " << m.nb_points());
+  
+
+//   // Test on the new structure
+//   exectime = gmm::uclock_sec();
+  
+//   bgeot::node_tab pttab3;
+
+//   for (dal::bv_visitor i(m.convex_index()); !i.finished(); ++i) {
+//     for (int j = 0; j <= dim; ++j)
+//       pttab3.add_node(m.points_of_convex(i)[j]);
+//   }
+//   cout << "Time to insert points in new structure: "
+//        << gmm::uclock_sec() - exectime << endl;
+//   GMM_ASSERT1(pttab3.card() == m.nb_points(),
+// 	      "Problem in identifying points " << pttab3.card()
+// 	      << " : " << m.nb_points());
+  
+
+
+//   // Test of a simple sort on one "random" componentwith Bkdtree
+//   exectime = gmm::uclock_sec();
+//   std::vector<base_node> T0;
+//   bgeot::kdtree trees[30];
+//   bgeot::kdtree_tab_type ipts;
+  
+//   for (dal::bv_visitor ic(m.convex_index()); !ic.finished(); ++ic) {
+//     for (int jp = 0; jp <= dim; ++jp) {
+
+//       // insert a point.
+//       T0.push_back(m.points_of_convex(ic)[jp]);
+
+//       if (T0.size() >= 4) {
+// 	size_type i = 0;
+// 	for (; i < 30; ++i)
+// 	  if (trees[i].nb_points() == 0) break;
+
+// 	// cout << "building tree " << i << endl;
+	
+// 	for (size_type j = 0; j < 4; ++j)
+// 	  trees[i].add_point(T0[j]);
+// 	T0.resize(0);
+
+// 	for (size_type j = 0; j < i; ++j) {
+// 	  for (size_type k = 0; k < trees[j].nb_points(); ++k)
+// 	    trees[i].add_point(trees[j].points()[k].n);
+// 	  trees[j].clear();
+// 	}
+// 	trees[i].points_in_box(ipts, base_node(dim), base_node(dim));
+//       }
+      
+
+//     }
+//   }
+
+//   cout << "Time to insert points in bkdtree: "
+//        << gmm::uclock_sec() - exectime << endl;
+
+
+
+  getfem::mesh_fem mf1(m);
+  mf1.set_finite_element( getfem::classical_fem(pgt, 1) );
+  cout << "nb points = " << m.nb_points()
+       << "nb dof = " << mf1.nb_dof() <<  endl;
+  GMM_ASSERT1(m.nb_points() == mf1.nb_dof(), "Problem in identifying dofs");
+
+  // shake a little bit the mesh
+  getfem::mesh::PT_TAB &pts = m.points();
+  for (size_type i = 0; i < m.nb_points(); ++i)
+    for (int k = 0; k < dim; ++k)
+      pts[i][k] += gmm::random(double()) * 1e-10;
+  getfem::mesh_fem mf2(m);
+  mf2.set_finite_element( getfem::classical_fem(pgt, 1) );
+  cout << "nb points = " << m.nb_points()
+       << "nb dof = " << mf2.nb_dof() <<  endl;
+  GMM_ASSERT1(m.nb_points() == mf2.nb_dof(), "Problem in identifying dofs");
+
+}
+
+
+void test_search_point() {
+  const char *s = "BEGIN POINTS LIST\n"
+    "  POINT  1  -4  6  2\n"
+    "  POINT  2  0  6  0\n"
+    "  POINT  3  0  2  0\n"
+    "  POINT  4  -2  6  2\n"
+    "  POINT  5  0  4  0\n"
+    "  POINT  6  -1.5  4.5  0.5\n"
+    "  POINT  7  1  2  0\n"
+    "  POINT  8  1.5  1.5  0\n"
+    "  POINT  9  5  5  0\n"
+    "  POINT  10  2  1  0\n"
+    "  POINT  11  6  3  0\n"
+    "  POINT  12  2  0  0\n"
+    "  POINT  13  6  0  0\n"
+    "  POINT  14  2  4  0\n"
+    "  POINT  15  4  2  0\n"
+    "  POINT  17  3  6  0\n"
+    "  POINT  18  2  -2  2\n"
+    "  POINT  19  2  -2  -2\n"
+    "  POINT  20  6  -2  2\n"
+    "  POINT  21  6  -2  -2\n"
+    "  POINT  22  2  -1  1\n"
+    "  POINT  23  2  -2.5  0\n"
+    "  POINT  24  2  -1  -1\n"
+    "  POINT  25  6  -1  1\n"
+    "  POINT  26  6  -2.5  0\n"
+    "  POINT  27  6  -1  -1\n"
+    "  POINT  28  -1  6  -1\n"
+    "  POINT  29  -1  2  -1\n"
+    "  POINT  30  1  6  -2\n"
+    "  POINT  31  1  2  -2\n"
+    "  POINT  32  0  6  -3\n"
+    "  POINT  33  0  2  -3\n"
+    "  POINT  34  2  -5  -2\n"
+    "  POINT  35  2  -4  0\n"
+    "  POINT  36  4  -5  2\n"
+    "  POINT  37  6  -5  -2\n"
+    "  POINT  38  6  -5  0\n"
+    "  POINT  46  4  4  0\n"
+    "  POINT  49  6  -5  2\n"
+    "\n"
+    "END POINTS LIST\n"
+    "\n"
+    "\n"
+    "\n"
+    "BEGIN MESH STRUCTURE DESCRIPTION\n"
+    "\n"
+    "CONVEX 0    'GT_PK(2,2)'      1  4  2  6  5  3\n"
+    "CONVEX 1    'GT_QK(2,1)'      2  17  3  7\n"
+    "CONVEX 2    'GT_QK(2,2)'      7  8  10  14  46  15  17  9  11\n"
+    "CONVEX 3    'GT_QK(2,1)'      10  12  11  13\n"
+    "CONVEX 4    'GT_PRODUCT(GT_PK(2,2),GT_PK(1,1))'      12  22  18  24  23  19  13  25  20  27  26  21\n"
+    "CONVEX 5    'GT_PRODUCT(GT_PK(1,1),GT_PK(1,3))'      2  3  28  29  30  31  32  33\n"
+    "CONVEX 8    'GT_PRODUCT(GT_PK(1,2),GT_PRODUCT(GT_PK(1,1),GT_PK(1,1)))'      19  23  18  21  26  20  34  35  36  37  38  49\n"
+    "\n"
+    "END MESH STRUCTURE DESCRIPTION\n";
+  std::stringstream ss(s);
+  getfem::mesh m; m.read_from_file(ss);
+  cout << "read " << m.nb_points() << " points and " << m.convex_index().card() << " convexes\n";
+  dal::bit_vector pid = m.points().index();
+  cout << "point index: " << pid << "\n";
+  for (dal::bv_visitor ii(pid); !ii.finished(); ++ii) {
+    base_node P = m.points()[ii];
+    size_type j = m.search_point(P);
+    cerr << "search point " << ii << ": " << P << " -> " << j << "\n";
+    assert(j == ii);
+  }
+  base_node P(m.dim()); gmm::fill_random(P);
+  assert(m.search_point(P) == size_type(-1));
+}
+
+int main(void) {
+
+  test_mesh_building(2, 100); 
+
+  getfem::mesh m1;
+  test_convex_ref();
+  test_convex_simplif();
+  test_mesh(m1);
+  test_convex_simplif();
+  test_convex_quality(0.2,0);
+  test_convex_quality(-0.2,0);
+  test_convex_quality(-0.01,-0.2);
+  test_region();
+
+  test_search_point();
+  
+  for (size_type d = 1; d <= 4 /* 6 */; ++d)
+    test_mesh_matching(d);
+  
+  test_refinable(2, 1);
+  test_refinable(2, 2);
+  test_refinable(2, 3);
+  test_refinable(3, 1);
+  test_refinable(3, 2);
+  test_refinable(3, 3);
+  
   return 0;
 }
 
